package cn.com.qmth.examcloud.starters.crypto.common;

import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 加、解密方案组合
 */
public class CryptoGroup {

    private List<CryptoItem> items;

    /**
     * 构造方案组合
     *
     * @param combination 组合名称
     */
    public CryptoGroup(String combination) {
        if (StringUtils.isEmpty(combination)) {
            return;
        }

        items = new ArrayList<>();
        for (int n = 0; n < combination.length(); n++) {
            Character c = combination.charAt(n);

            CryptoType type = CryptoType.findByName(c.toString());
            if (type == null) {
                throw new CryptoException(CryptoConstant.ALGORITHM_INVALID);
            }

            items.add(new CryptoItem(type));
        }
    }

    /**
     * 匹配算法的密钥值（特殊场景：所有算法都共用一个密码！！！）
     *
     * @param keyValue 密钥值
     * @return
     */
    public CryptoGroup matchKeys(String keyValue) {
        for (CryptoItem item : getItems()) {
            Key key = new Key(keyValue, KeyType.COMMON);
            item.setEncryptKey(key);
            item.setDecryptKey(key);
        }

        return this;
    }

    /**
     * 匹配某种算法的密钥值（适用加、解密密码都一样场景）
     *
     * @param algorithm 算法
     * @param keyValue  密钥值
     * @return
     */
    public CryptoGroup matchKeys(Algorithm algorithm, String keyValue) {
        if (StringUtils.isEmpty(keyValue)) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        Key key = new Key(keyValue, KeyType.COMMON);
        for (CryptoItem item : getItems()) {
            if (item.getType().getAlgorithm() == algorithm) {
                item.setEncryptKey(key);
                item.setDecryptKey(key);
            }
        }

        return this;
    }

    /**
     * 匹配某种算法的密钥值（适用加、解密密码不一样场景）
     *
     * @param algorithm  算法
     * @param encryptKey 加密密钥值
     * @param decryptKey 解密密钥值
     * @return
     */
    public CryptoGroup matchKeys(Algorithm algorithm, Key encryptKey, Key decryptKey) {
        if (encryptKey.getType() == null || StringUtils.isEmpty(encryptKey.getValue())
                || decryptKey.getType() == null || StringUtils.isEmpty(decryptKey.getValue())) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        for (CryptoItem item : getItems()) {
            if (item.getType().getAlgorithm() == algorithm) {
                item.setEncryptKey(encryptKey);
                item.setDecryptKey(decryptKey);
            }
        }

        return this;
    }

    public List<CryptoItem> getItems() {
        if (items == null) {
            items = new ArrayList<>();
        }
        return items;
    }

    public List<CryptoItem> getReverseItems() {
        // 倒序
        List<CryptoItem> list = new ArrayList<>();
        for (int n = getItems().size() - 1; n >= 0; n--) {
            list.add(getItems().get(n));
        }
        return list;
    }

}
