package cn.com.qmth.examcloud.starters.crypto.service;

import cn.com.qmth.examcloud.starters.crypto.common.*;
import cn.com.qmth.examcloud.starters.crypto.service.impl.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 加、解密算法工厂类
 */
public class CryptoFactory {

    private static final Logger log = LoggerFactory.getLogger(CryptoFactory.class);

    public CryptoService getCryptoService(CryptoType type) {
        if (type.getImpl() == AesCryptoService.class) {
            return new AesCryptoService();
        } else if (type.getImpl() == DesCryptoService.class) {
            return new DesCryptoService();
        } else if (type.getImpl() == TripleDesCryptoService.class) {
            return new TripleDesCryptoService();
        } else if (type.getImpl() == Rc4CryptoService.class) {
            return new Rc4CryptoService();
        } else if (type.getImpl() == RsaCryptoService.class) {
            return new RsaCryptoService();
        } else if (type.getImpl() == Base64CryptoService.class) {
            return new Base64CryptoService();
        }

        throw new CryptoException(CryptoConstant.ALGORITHM_INVALID);
    }

    /**
     * 按加密方案组合“顺序”依次加密
     */
    public String encrypt(CryptoGroup cryptoGroup, String str) {
        return this.encrypt(cryptoGroup, str, false);
    }

    /**
     * 按加密方案组合“顺序或倒序”依次加密
     */
    public String encrypt(CryptoGroup cryptoGroup, String str, boolean reverse) {
        if (StringUtils.isEmpty(str)) {
            log.warn("[encrypt] input is empty...");
            throw new CryptoException(CryptoConstant.REQUEST_PARAM_ERROR);
        }

        List<CryptoItem> items;
        if (reverse) {
            // 倒序
            items = cryptoGroup.getReverseItems();
        } else {
            items = cryptoGroup.getItems();
        }

        for (CryptoItem item : items) {
            str = this.getCryptoService(item.getType()).encrypt(str, item.getEncryptKey());
        }

        return str;
    }

    /**
     * 按加密方案组合“顺序”依次解密
     */
    public String decrypt(CryptoGroup cryptoGroup, String str) {
        return this.decrypt(cryptoGroup, str, false);
    }

    /**
     * 按加密方案组合“顺序或倒序”依次解密
     */
    public String decrypt(CryptoGroup cryptoGroup, String str, boolean reverse) {
        if (StringUtils.isEmpty(str)) {
            log.warn("[decrypt] input is empty...");
            throw new CryptoException(CryptoConstant.REQUEST_PARAM_ERROR);
        }

        List<CryptoItem> items;
        if (reverse) {
            // 倒序
            items = cryptoGroup.getReverseItems();
        } else {
            items = cryptoGroup.getItems();
        }

        for (CryptoItem item : items) {
            str = this.getCryptoService(item.getType()).decrypt(str, item.getDecryptKey());
        }

        return str;
    }

}
