package cn.com.qmth.examcloud.starters.crypto.utils;

import cn.com.qmth.examcloud.starters.crypto.common.CryptoConstant;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoException;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

/**
 * AES加、解密
 */
public class AesUtil {

    private static final Logger log = LoggerFactory.getLogger(AesUtil.class);

    /**
     * 算法
     */
    private static final String ALGORITHM = "AES";

    /**
     * ECB模式
     * 算法/模式/补码方式
     */
    private static final String ECB_PKCS5Padding = "AES/ECB/PKCS5Padding";

    /**
     * CBC模式
     * 算法/模式/补码方式
     */
    private static final String CBC_PKCS5Padding = "AES/CBC/PKCS5Padding";

    /**
     * 默认密钥长度
     */
    private static final int DEFAULT_KEY_SIZE = 128;

    /**
     * 默认偏移向量字符串（长度必须16位，仅CBC模式下使用）
     */
    private static final String DEFAULT_IV = "@M#A$G%A^2&0*2(1";

    /**
     * 加密
     */
    public static String encrypt(String str, byte[] key) {
        return encrypt(str, key, true);
    }

    /**
     * 加密
     */
    public static String encrypt(String str, byte[] key, boolean cbcMode) {
        if (StringUtils.isEmpty(str)) {
            return "";
        }

        try {
            SecretKey secretKey = initSecretKey(key);
            if (log.isDebugEnabled()) {
                log.debug("Key:" + new String(secretKey.getEncoded()));
            }

            Cipher cipher;
            if (cbcMode) {
                cipher = Cipher.getInstance(CBC_PKCS5Padding);
                cipher.init(Cipher.ENCRYPT_MODE, secretKey, initIv(key));
            } else {
                cipher = Cipher.getInstance(ECB_PKCS5Padding);
                cipher.init(Cipher.ENCRYPT_MODE, secretKey);
            }

            byte[] bytes = str.getBytes(StandardCharsets.UTF_8);
            byte[] result = cipher.doFinal(bytes);

            // return Base64.encodeBase64String(result);
            return Hex.encodeHexString(result);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new CryptoException(CryptoConstant.ENCRYPT_ERROR, e);
        }
    }

    /**
     * 解密
     */
    public static String decrypt(String str, byte[] key) {
        return decrypt(str, key, true);
    }

    /**
     * 解密
     */
    public static String decrypt(String str, byte[] key, boolean cbcMode) {
        if (StringUtils.isEmpty(str)) {
            return "";
        }

        try {
            SecretKey secretKey = initSecretKey(key);
            if (log.isDebugEnabled()) {
                log.debug("Key:" + new String(secretKey.getEncoded()));
            }

            Cipher cipher;
            if (cbcMode) {
                cipher = Cipher.getInstance(CBC_PKCS5Padding);
                cipher.init(Cipher.DECRYPT_MODE, secretKey, initIv(key));
            } else {
                cipher = Cipher.getInstance(ECB_PKCS5Padding);
                cipher.init(Cipher.DECRYPT_MODE, secretKey);
            }

            // byte[] bytes = Base64.decodeBase64(str);
            byte[] bytes = Hex.decodeHex(str);

            byte[] result = cipher.doFinal(bytes);
            return new String(result, StandardCharsets.UTF_8);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new CryptoException(CryptoConstant.DECRYPT_ERROR, e);
        }
    }

    /**
     * 随机生成密钥
     */
    public static byte[] randomKey() {
        try {
            KeyGenerator keyGenerator = KeyGenerator.getInstance(ALGORITHM);

            // 密钥长度可选：128、192、256
            // 其中192和256，需获得“无政策限制权限文件”，针对JDK版本对应覆盖local_policy.jar和US_export_policy.jar两个文件！
            keyGenerator.init(DEFAULT_KEY_SIZE);

            SecretKey secretKey = keyGenerator.generateKey();
            return secretKey.getEncoded();
        } catch (NoSuchAlgorithmException e) {
            log.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    public static SecretKey initSecretKey(byte[] key) {
        if (key.length == 16 || key.length == 24 || key.length == 32) {
            // key长度范围为 16位 或 24位 或 32位
            return new SecretKeySpec(key, ALGORITHM);
        }

        // 注：其它key长度时统一转为16位（长度不够时末尾补0，超出则截取）
        byte[] newKey = Arrays.copyOf(key, 16);
        return new SecretKeySpec(newKey, ALGORITHM);
    }

    public static IvParameterSpec initIv(byte[] key) {
        byte[] ivBytes = DEFAULT_IV.getBytes(StandardCharsets.UTF_8);
        // 生成新的向量值 = 默认向量值的前8位 + key值的前8位
        // byte[] newKey = Arrays.copyOf(key, 16);
        // System.arraycopy(newKey, 0, ivBytes, 8, Math.min(newKey.length, 8));
        return new IvParameterSpec(ivBytes);
    }

}
