package cn.com.qmth.examcloud.starters.crypto.utils;

import cn.com.qmth.examcloud.starters.crypto.common.CryptoConstant;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

/**
 * RC4加、解密
 */
public class Rc4Util {

    private static final Logger log = LoggerFactory.getLogger(Rc4Util.class);

    /**
     * 算法
     */
    private static final String ALGORITHM = "RC4";

    /**
     * 加密
     */
    public static String encrypt(String str, String key) {
        if (StringUtils.isEmpty(str)) {
            return "";
        }

        try {
            SecretKey secretKey = initSecretKey(key);
            if (log.isDebugEnabled()) {
                log.debug("Key:" + new String(secretKey.getEncoded()));
            }

            Cipher cipher = Cipher.getInstance(ALGORITHM);
            cipher.init(Cipher.ENCRYPT_MODE, secretKey);

            byte[] bytes = str.getBytes(StandardCharsets.UTF_8);
            byte[] result = cipher.update(bytes);
            // byte[] result = rc4Cipher(bytes, key);

            return Base64.encodeBase64String(result);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new CryptoException(CryptoConstant.ENCRYPT_ERROR, e);
        }
    }

    /**
     * 解密
     */
    public static String decrypt(String str, String key) {
        if (StringUtils.isEmpty(str)) {
            return "";
        }

        try {
            SecretKey secretKey = initSecretKey(key);
            if (log.isDebugEnabled()) {
                log.debug("Key:" + new String(secretKey.getEncoded()));
            }

            Cipher cipher = Cipher.getInstance(ALGORITHM);
            cipher.init(Cipher.DECRYPT_MODE, secretKey);

            byte[] bytes = Base64.decodeBase64(str);
            byte[] result = cipher.update(bytes);
            // byte[] result = rc4Cipher(bytes, key);

            return new String(result, StandardCharsets.UTF_8);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new CryptoException(CryptoConstant.DECRYPT_ERROR, e);
        }
    }

    public static SecretKey initSecretKey(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        byte[] keyBytes = key.getBytes(StandardCharsets.UTF_8);
        if (keyBytes.length >= 5 && keyBytes.length <= 128) {
            // key长度范围为 区间（5位，128位）
            return new SecretKeySpec(keyBytes, ALGORITHM);
        }

        // 注：其它key长度时统一转为16位（长度不够时末尾补0，超出则截取）
        byte[] newKey = Arrays.copyOf(keyBytes, 16);
        return new SecretKeySpec(newKey, ALGORITHM);
    }

    private static byte[] rc4Cipher(byte[] data, String key) {
        int xorIndex, x = 0, y = 0;
        byte[] keyBytes = rc4Key(key);

        byte[] result = new byte[data.length];
        for (int i = 0; i < data.length; i++) {
            x = (x + 1) & 0xff;
            y = ((keyBytes[x] & 0xff) + y) & 0xff;

            byte tmp = keyBytes[x];
            keyBytes[x] = keyBytes[y];
            keyBytes[y] = tmp;

            xorIndex = ((keyBytes[x] & 0xff) + (keyBytes[y] & 0xff)) & 0xff;
            result[i] = (byte) (data[i] ^ keyBytes[xorIndex]);
        }

        return result;
    }

    private static byte[] rc4Key(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        byte[] result = new byte[256];
        for (int i = 0; i < 256; i++) {
            result[i] = (byte) i;
        }

        int index1 = 0, index2 = 0;
        byte[] keyBytes = key.getBytes();
        for (int i = 0; i < 256; i++) {
            index2 = ((keyBytes[index1] & 0xff) + (result[i] & 0xff) + index2) & 0xff;

            byte tmp = result[i];
            result[i] = result[index2];
            result[index2] = tmp;

            index1 = (index1 + 1) % keyBytes.length;
        }

        return result;
    }

}
