package cn.com.qmth.examcloud.starters.face.verify.model.faceplus;

import cn.com.qmth.examcloud.starters.face.verify.FaceVerifyProperties;
import cn.com.qmth.examcloud.starters.face.verify.common.Constants;
import cn.com.qmth.examcloud.starters.face.verify.common.FaceVerifyException;
import cn.com.qmth.examcloud.starters.face.verify.common.HttpClientBuilder;
import cn.com.qmth.examcloud.starters.face.verify.common.JsonHelper;
import cn.com.qmth.examcloud.starters.face.verify.model.FaceResult;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import okhttp3.FormBody;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Map;

public class FacePlusApiHelper {

    private final static Logger log = LoggerFactory.getLogger(FacePlusApiHelper.class);

    public static FaceResult faceDetect(FaceVerifyProperties properties, Map<String, String> params) {
        if (StringUtils.isBlank(properties.getFacePlusKey())) {
            throw new IllegalArgumentException("[FACE++] apiKey must be not empty.");
        }

        if (StringUtils.isBlank(properties.getFacePlusSecret())) {
            throw new IllegalArgumentException("[FACE++] apiSecret must be not empty.");
        }

        FormBody.Builder formBody = new FormBody.Builder();
        formBody.add("api_key", properties.getFacePlusKey());
        formBody.add("api_secret", properties.getFacePlusSecret());

        for (Map.Entry<String, String> entry : params.entrySet()) {
            formBody.add(entry.getKey(), entry.getValue());
        }

        Request.Builder request = new Request.Builder().url(Constants.FACEPP_FACE_DETECT_URL).post(formBody.build());
        FacePlusResponse response = callApi(Constants.FACEPP_FACE_DETECT_URL, request);

        FaceResult result = new FaceResult();
        result.setFacePass(false);
        result.setFaceScore(0d);
        result.setFaceNum(0);
        result.setJsonResult(new JsonHelper().style(Include.NON_NULL).toJson(response));

        String errorMessage = response.getError_message();
        if (StringUtils.isNotEmpty(errorMessage)) {
            result.setApiNeedRetry(true);// 默认值
            result.setError(errorMessage);

            if (errorMessage.startsWith("INVALID_IMAGE_SIZE")
                    || errorMessage.startsWith("INVALID_IMAGE_URL")
                    || errorMessage.startsWith("IMAGE_FILE_TOO_LARGE")) {
                // INVALID_IMAGE_SIZE 上传的图像像素尺寸太大或太小
                // INVALID_IMAGE_URL 图片URL错误或者无效
                // IMAGE_FILE_TOO_LARGE 图像文件太大，图片文件大小不超过2MB

                result.setApiNeedRetry(false);
            }

            // 请求并发超限(约20个/秒，可按需调整)
            /*if (errorMessage.startsWith("CONCURRENCY_LIMIT_EXCEEDED")) {
                log.warn("facepp_api_limit...");
            }*/

            return result;
        }

        int faceNum = response.getFace_num() != null ? response.getFace_num() : 0;
        result.setFaceNum(faceNum);

        if (faceNum == 1) {
            // 是否人脸检测通过 (只有一张人脸算成功，否则算失败)
            result.setFacePass(true);
        } else {
            result.setError("faceNum=" + faceNum + " but notEq 1");
        }

        return result;
    }

    public static FaceResult faceCompare(FaceVerifyProperties properties, Map<String, String> params) {
        if (StringUtils.isBlank(properties.getFacePlusKey())) {
            throw new IllegalArgumentException("[FACE++] apiKey must be not empty.");
        }

        if (StringUtils.isBlank(properties.getFacePlusSecret())) {
            throw new IllegalArgumentException("[FACE++] apiSecret must be not empty.");
        }

        FormBody.Builder formBody = new FormBody.Builder();
        formBody.add("api_key", properties.getFacePlusKey());
        formBody.add("api_secret", properties.getFacePlusSecret());

        for (Map.Entry<String, String> entry : params.entrySet()) {
            formBody.add(entry.getKey(), entry.getValue());
        }

        Request.Builder request = new Request.Builder().url(Constants.FACEPP_FACE_COMPARE_URL).post(formBody.build());
        FacePlusResponse response = callApi(Constants.FACEPP_FACE_COMPARE_URL, request);

        FaceResult result = new FaceResult();
        result.setFacePass(false);
        result.setFaceScore(0d);
        result.setFaceNum(0);
        result.setJsonResult(new JsonHelper().style(Include.NON_NULL).toJson(response));

        String errorMessage = response.getError_message();
        if (StringUtils.isNotEmpty(errorMessage)) {
            result.setApiNeedRetry(true);// 默认值
            result.setError(errorMessage);

            if (errorMessage.startsWith("INVALID_IMAGE_SIZE")
                    || errorMessage.startsWith("INVALID_IMAGE_URL")
                    || errorMessage.startsWith("IMAGE_FILE_TOO_LARGE")) {
                // INVALID_IMAGE_SIZE 上传的图像像素尺寸太大或太小
                // INVALID_IMAGE_URL 图片URL错误或者无效
                // IMAGE_FILE_TOO_LARGE 图像文件太大，图片文件大小不超过2MB

                result.setApiNeedRetry(false);
            }

            // 请求并发超限(约20个/秒，可按需调整)
            /*if (errorMessage.startsWith("CONCURRENCY_LIMIT_EXCEEDED")) {
                log.warn("facepp_api_limit...");
            }*/

            return result;
        }

        FacePlusThresholds thresholds = response.getThresholds();
        if (thresholds != null) {
            float confidence = response.getConfidence() != null ? response.getConfidence() : 0f;
            result.setFaceScore(confidence);

            // 是否人脸对比通过
            if (confidence >= thresholds.getLe_4()) {
                result.setFacePass(true);
            } else {
                result.setError("faceConfidence=" + confidence + " but less than " + thresholds.getLe_4());
            }

            if (response.getFaces2() != null) {
                // 待比对照片中出现几张人脸数（用于判断是否存在陌生人）
                result.setFaceNum(response.getFaces2().size());
            }
        }

        return result;
    }

    private static FacePlusResponse callApi(String requestUrl, Request.Builder request) {
        long start = System.currentTimeMillis();
        try (Response response = HttpClientBuilder.getClient().newCall(request.build()).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body != null ? body.string() : null;
            int code = response.code();

            long cost = System.currentTimeMillis() - start;
            // System.out.println("response:" + code + " cost:" + cost + "ms body:" + bodyStr);

            if (response.isSuccessful() || code == 400 || code == 401 || code == 403 || code == 412) {
                FacePlusResponse result = new JsonHelper().parseJson(bodyStr, FacePlusResponse.class);
                if (result != null) {
                    return result;
                }
            }

            if (code == 413) {
                // 请求大小超过了2MB限制。该错误的返回格式为纯文本，不是json格式。
                FacePlusResponse result = new FacePlusResponse();
                result.setError_message("IMAGE_FILE_TOO_LARGE");
                return result;
            }

            log.warn("[FACE++] url:{} response:{} cost:{}ms body:{}", requestUrl, code, cost, bodyStr);
        } catch (IOException e) {
            log.error("[FACE++] url:{} {}:{}", requestUrl, e.getClass().getSimpleName(), e.getMessage());
        }

        throw new FaceVerifyException("人脸接口调用异常！");
    }

}
