/*
 * Copyright (c) 2020 QMTH All Rights Reserved.
 * Created by Deason on 2020-07-22 16:19:27
 */

package cn.com.qmth.examcloud.starters.greetest.service.impl;

import cn.com.qmth.examcloud.starters.greetest.GeetestProperties;
import cn.com.qmth.examcloud.starters.greetest.model.RegisterReq;
import cn.com.qmth.examcloud.starters.greetest.model.RegisterResp;
import cn.com.qmth.examcloud.starters.greetest.model.ValidateReq;
import cn.com.qmth.examcloud.starters.greetest.model.ValidateResp;
import cn.com.qmth.examcloud.starters.greetest.service.GeetestService;
import cn.com.qmth.examcloud.starters.greetest.service.GeetestSessionManager;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import okhttp3.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * 极验验证码相关接口
 */
public class GeetestServiceImpl implements GeetestService {

    private static final Logger LOG = LoggerFactory.getLogger(GeetestServiceImpl.class);

    private GeetestProperties properties;

    private GeetestSessionManager sessionManager;

    /**
     * 验证初始化
     */
    @Override
    public RegisterResp register(RegisterReq req) {
        if (StringUtils.isEmpty(req.getUser_id())) {
            return new RegisterResp(false, "参数“user_id”值不能为空");
        }

        String params = this.buildRegisterParams(req);
        LOG.info("[Geetest register] " + params);
        String requestUrl = properties.getApiUrl() + "/register.php" + params;

        String challenge = null;
        Request.Builder request = new Request.Builder().url(requestUrl).get();
        try (Response response = this.httpClient().newCall(request.build()).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body.string();
            LOG.info("[Geetest register] response:" + response.code() + " body:" + bodyStr);

            if (response.isSuccessful()) {
                challenge = this.parseJsonValue(bodyStr, "challenge");
            }
        } catch (Exception e) {
            LOG.error("[Geetest register] error:" + e.getMessage(), e);
        }

        // 封装结果
        RegisterResp resp = new RegisterResp();
        resp.setNew_captcha(true);
        resp.setGt(properties.getId());
        if (StringUtils.isEmpty(challenge) || "0".equals(challenge)) {
            LOG.info("请求极验register接口失败，后续流程走宕机模式！user_id = " + req.getUser_id());
            resp.setChallenge(UUID.randomUUID().toString().replaceAll("-", ""));
            resp.setSuccess(false);
        } else {
            // 默认实现MD5签名算法
            resp.setChallenge(DigestUtils.md5Hex(challenge + properties.getKey()));
            resp.setSuccess(true);
        }

        // 设置会话信息
        sessionManager.setSession(req.getUser_id(), resp.getSuccess());

        return resp;
    }

    /**
     * 二次验证
     */
    @Override
    public ValidateResp validate(ValidateReq req) {
        if (StringUtils.isEmpty(req.getUser_id())) {
            return new ValidateResp(false, "参数“user_id”值不能为空");
        }
        if (StringUtils.isEmpty(req.getChallenge())) {
            return new ValidateResp(false, "参数“challenge”值不能为空");
        }
        if (StringUtils.isEmpty(req.getValidate())) {
            return new ValidateResp(false, "参数“validate”值不能为空");
        }
        if (StringUtils.isEmpty(req.getSeccode())) {
            return new ValidateResp(false, "参数“seccode”值不能为空");
        }

        // 校验会话信息
        Boolean status = sessionManager.getSession(req.getUser_id());
        if (status == null) {
            LOG.info("验证码已过期，请重试！user_id = " + req.getUser_id());
            return new ValidateResp(false, "验证码已过期，请重试！");
        }

        if (status == false) {
            LOG.info("宕机通过！user_id = " + req.getUser_id());
            return new ValidateResp(true, "宕机通过");
        }

        String params = this.buildValidateParams(req);
        LOG.info("[Geetest validate] " + params);
        String requestUrl = properties.getApiUrl() + "/validate.php" + params;

        String seccode = null;
        RequestBody formBody = new FormBody.Builder(Charset.forName("UTF-8")).build();
        Request.Builder request = new Request.Builder().url(requestUrl).post(formBody);
        try (Response response = this.httpClient().newCall(request.build()).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body.string();
            LOG.info("[Geetest validate] response:" + response.code() + " body:" + bodyStr);

            if (response.isSuccessful()) {
                seccode = this.parseJsonValue(bodyStr, "seccode");
            }
        } catch (Exception e) {
            LOG.error("[Geetest validate] error:" + e.getMessage(), e);
        }

        // 封装结果
        ValidateResp resp = new ValidateResp();
        if (StringUtils.isEmpty(seccode) || "false".equals(seccode)) {
            resp.setSuccess(false);
            resp.setMsg("验证不通过");
            resp.setVersion(properties.getSdk());

            // 清理会话信息
            sessionManager.removeSession(req.getUser_id());
        } else {
            resp.setSuccess(true);
            resp.setMsg("验证通过");
        }

        return resp;
    }

    private String buildRegisterParams(RegisterReq req) {
        StringBuilder params = new StringBuilder();

        try {
            final String charset = "UTF-8";
            params.append("?").append("user_id").append("=").append(StringUtils.isNotEmpty(req.getUser_id())
                    ? URLEncoder.encode(req.getUser_id(), charset) : "");
            params.append("&").append("client_type").append("=").append(StringUtils.isNotEmpty(req.getClient_type())
                    ? URLEncoder.encode(req.getClient_type(), charset) : properties.getClient_type());
            params.append("&").append("ip_address").append("=").append(StringUtils.isNotEmpty(req.getIp_address())
                    ? URLEncoder.encode(req.getIp_address(), charset) : "");

            params.append("&").append("digestmod").append("=").append(URLEncoder.encode(properties.getDigestmod(), charset));
            params.append("&").append("gt").append("=").append(URLEncoder.encode(properties.getId(), charset));
            params.append("&").append("json_format").append("=").append(URLEncoder.encode(properties.getJson_format(), charset));
            params.append("&").append("sdk").append("=").append(URLEncoder.encode(properties.getSdk(), charset));
        } catch (Exception e) {
            // ignore
        }

        return params.toString();
    }

    private String buildValidateParams(ValidateReq req) {
        StringBuilder params = new StringBuilder();

        try {
            final String charset = "UTF-8";
            params.append("?").append("user_id").append("=").append(StringUtils.isNotEmpty(req.getUser_id())
                    ? URLEncoder.encode(req.getUser_id(), charset) : "");
            params.append("&").append("client_type").append("=").append(StringUtils.isNotEmpty(req.getClient_type())
                    ? URLEncoder.encode(req.getClient_type(), charset) : properties.getClient_type());
            params.append("&").append("ip_address").append("=").append(StringUtils.isNotEmpty(req.getIp_address())
                    ? URLEncoder.encode(req.getIp_address(), charset) : "");

            params.append("&").append("seccode").append("=").append(URLEncoder.encode(req.getSeccode(), charset));
            params.append("&").append("challenge").append("=").append(URLEncoder.encode(req.getChallenge(), charset));
            params.append("&").append("captchaid").append("=").append(URLEncoder.encode(properties.getId(), charset));
            params.append("&").append("json_format").append("=").append(URLEncoder.encode(properties.getJson_format(), charset));
            params.append("&").append("sdk").append("=").append(URLEncoder.encode(properties.getSdk(), charset));
        } catch (Exception e) {
            // ignore
        }

        return params.toString();
    }

    public void setProperties(GeetestProperties properties) {
        this.properties = properties;
    }

    public void setSessionManager(GeetestSessionManager sessionManager) {
        this.sessionManager = sessionManager;
    }

    private OkHttpClient httpClient() {
        return new OkHttpClient
                .Builder()
                .connectTimeout(15, TimeUnit.SECONDS)
                .readTimeout(15, TimeUnit.SECONDS)
                .writeTimeout(15, TimeUnit.SECONDS)
                .build();
    }

    private String parseJsonValue(String json, String fieldName) {
        try {
            JsonNode jsonNode = new ObjectMapper().readTree(json);
            return jsonNode.get(fieldName).asText();
        } catch (Exception e) {
            LOG.warn(String.format("[%s] json value not exist...", fieldName));
        }
        return null;
    }

}