package cn.com.qmth.examcloud.commons.helpers;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 试题选项辅助工具类
 */
public class QuestionOptionHelper {

    private static final Logger log = LoggerFactory.getLogger(QuestionOptionHelper.class);

    /**
     * 将(0-25)之间的数字转换成大写字母
     */
    public static String numberToLetter(int number) {
        if (number >= 0 && number <= 25) {
            // 只处理0-25之间的数字
            char c = (char) (65 + number);
            return String.valueOf(c);
        }

        return null;
    }

    /**
     * 将字符串中包含的“大写字母”全部对应转换成(0-25)之间的数字
     */
    public static List<Integer> letterToNumber(String letters) {
        List<Integer> numbers = new ArrayList<>();

        if (StringUtils.isNotEmpty(letters)) {
            char[] chars = letters.toCharArray();
            for (char c : chars) {
                if (String.valueOf(c).matches("[A-Z]")) {
                    // 只处理A-Z之间的大写字母
                    numbers.add(c - 65);
                }
            }
        }

        return numbers;
    }

    /**
     * 解析数字编号的选项数据
     * 注：前期题库的选择题选项只支持0-9共十个选项，现需要扩充至最多支持26个选项
     * 为了兼容旧数据需要转换为Json数字数组，如：0123456789 --> [0,1,2,3,4,5,6,7,8,9]
     */
    public static List<Integer> parseNumberOptions(String numberStr) {
        List<Integer> numbers = new ArrayList<>();
        if (StringUtils.isEmpty(numberStr)) {
            return numbers;
        }

        if (numberStr.startsWith("[") && numberStr.endsWith("]")) {
            // 已经是Json数字数组的新格式数据，直接解析并返回
            String[] values = numberStr
                    .replace("[", "")
                    .replace("]", "")
                    .split(",");

            for (String x : values) {
                if (StringUtils.isBlank(x)) {
                    continue;
                }

                try {
                    numbers.add(Integer.parseInt(x.trim()));
                } catch (NumberFormatException e) {
                    log.warn("{}不可转换为数字！", x);
                }
            }

            return numbers;
        }

        // 旧数据格式
        char[] chars = numberStr.toCharArray();
        for (char c : chars) {
            if (String.valueOf(c).matches("[0-9]")) {
                // 只处理0-9之间的数字
                numbers.add(c - 48);
            }
        }

        return numbers;
    }

    /**
     * 解析数字，返回JSON数字数组的字符串
     */
    public static String parseNumbers(String numberStr) {
        List<Integer> numbers = parseNumberOptions(numberStr);
        // 从小到大排序
        Collections.sort(numbers);
        return "[" + StringUtils.join(numbers, ",") + "]";
    }

    /**
     * 比较正确答案和学生作答答案是否一致（兼容新、旧格式数据）
     */
    public static boolean isEqualAnswer(String correctAnswer, String studentAnswer) {
        List<Integer> correctAnswers = parseNumberOptions(correctAnswer);
        List<Integer> studentAnswers = parseNumberOptions(studentAnswer);
        return CollectionUtils.isEqualCollection(correctAnswers, studentAnswers);
    }

    /*public static void main(String[] args) {
        System.out.println(parseNumbers("987654321 "));
        System.out.println(parseNumbers("[3,22,1 ]"));

        System.out.println(isEqualAnswer("123", "321"));
        System.out.println(isEqualAnswer("123", "[3,2,1]"));
        System.out.println(isEqualAnswer("123", "[321]"));
    }*/

}
