package cn.com.qmth.examcloud.commons.helpers.concurrency.simple;

import cn.com.qmth.examcloud.commons.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 并发任务
 *
 * @author WANGWEI
 * @date 2019年6月19日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
public class ConcurrentTask<T> {

    private static final Logger LOG = LoggerFactory.getLogger(ConcurrentTask.class);

    private BlockingQueue<T> queue = new LinkedBlockingQueue<T>(10000);

    private BlockingQueue<Integer> workerMessages = new LinkedBlockingQueue<Integer>(10000);

    private final WorkerController workerController = new WorkerController();

    /**
     * 最大线程数
     */
    private int maxActiveThreadSize = 5;

    /**
     * 不死线程数
     */
    private int minThreadSize = 2;

    private ThreadPoolExecutor threadPoolExecutor;

    /**
     * 处理者
     */
    private Worker<T> worker;

    /**
     * 巡检周期
     */
    private int inspectionPeriod = 30;

    /**
     * 任务名称
     */
    private String taskName;

    /**
     * 构造函数
     *
     * @param taskName
     */
    public ConcurrentTask(String taskName) {
        super();
        this.taskName = taskName;
    }

    /**
     * 添加处理元素
     *
     * @param e
     * @return
     * @author WANGWEI
     */
    public boolean offerElement(T e) {
        boolean offer = queue.offer(e);
        if (LOG.isDebugEnabled()) {
            LOG.debug("offerElement. result=" + offer + ";element=" + e.toString());
        }
        return offer;
    }

    /**
     * 启动任务
     *
     * @author WANGWEI
     */
    public void start() {
        threadPoolExecutor = (ThreadPoolExecutor) Executors.newFixedThreadPool(maxActiveThreadSize);

        // 创建不死线程
        for (int i = 0; i < minThreadSize; i++) {
            addWorkerThread(true);
        }

        // 巡检线程
        Thread inspectionThread = new Thread() {

            @Override
            public void run() {
                Util.sleep(inspectionPeriod);

                while (true) {

                    int size = queue.size();
                    int activeCount = threadPoolExecutor.getActiveCount();
                    int warnCount = workerController.getWarnCount();

                    if (LOG.isInfoEnabled()) {
                        LOG.info("taskName [" + taskName + "]. activeCount = " + activeCount
                                + "; warnCount = " + warnCount);
                    }

                    // 巡检周期内(因并发超出限制导致的)警告数量未超过100时,增加一个worker
                    if (warnCount <= 100) {
                        if (100 < size && maxActiveThreadSize > activeCount) {
                            addWorkerThread(false);
                        }
                    } else {
                        // 巡检周期内(因并发超出限制导致的)警告数量超过100时,减少一个worker
                        workerMessages.offer(warnCount);
                    }

                    workerController.resetWarnCount();
                    Util.sleep(inspectionPeriod);
                }

            }

            ;

        };

        inspectionThread.setDaemon(true);
        inspectionThread.start();
    }

    /**
     * 添加处理线程
     *
     * @param immortal
     * @author WANGWEI
     */
    private void addWorkerThread(final boolean immortal) {
        LOG.info("create a new worker. immortal=" + immortal);
        Thread thread = new Thread() {

            long nullTimes = 0;

            @Override
            public void run() {
                while (true) {
                    T el = queue.poll();
                    if (null == el) {
                        nullTimes++;
                        if (10 <= nullTimes) {
                            if (immortal) {
                                Util.sleep(2);
                                continue;
                            } else {
                                LOG.info("no element.worker exist.");
                                break;
                            }
                        } else {
                            Util.sleep(2);
                            continue;
                        }
                    } else {
                        nullTimes = 0;
                    }

                    try {
                        if (LOG.isDebugEnabled()) {
                            LOG.debug("process. element=" + el.toString());
                        }
                        worker.process(workerController, el);
                    } catch (Exception e) {
                        LOG.error("unexpected exception", e);
                    }

                    // 非不死线程在抢到终止消息时,结束线程
                    if (!immortal) {
                        Integer warnCount = workerMessages.poll();
                        if (null != warnCount) {
                            LOG.info("worker exist. warnCount=" + warnCount);
                            break;
                        }
                    }

                }
            }
        };

        threadPoolExecutor.execute(thread);
    }

    public void setMaxActiveThreadSize(int maxActiveThreadSize) {
        this.maxActiveThreadSize = maxActiveThreadSize;
    }

    public void setMinThreadSize(int minThreadSize) {
        this.minThreadSize = minThreadSize;
    }

    public void setWorker(Worker<T> worker) {
        this.worker = worker;
    }

}
