package cn.com.qmth.examcloud.web.baidu;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.JsonHttpResponseHolder;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * baidu 客户端
 *
 * @author WANGWEI
 * @date 2019年9月16日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class BaiduClient {

    private static final Logger LOG = LoggerFactory.getLogger(BaiduClient.class);

    private static CloseableHttpClient httpclient;

    private static RequestConfig requestConfig;

    private static BaiduClient baiduClient;

    private static String apiKey;

    private static String secretKey;

    private BaiduClient() {
    }

    /**
     * 获取单例
     *
     * @return
     * @author WANGWEI
     */
    public static BaiduClient getClient() {
        if (null == baiduClient) {
            synchronized (BaiduClient.class) {
                if (null == baiduClient) {
                    baiduClient = new BaiduClient();
                }
            }
        }

        return baiduClient;
    }

    static {
        PoolingHttpClientConnectionManager cm = new PoolingHttpClientConnectionManager(60,
                TimeUnit.SECONDS);
        cm.setValidateAfterInactivity(1000);
        cm.setMaxTotal(8000);
        cm.setDefaultMaxPerRoute(200);

        requestConfig = RequestConfig.custom().setConnectionRequestTimeout(500)
                .setSocketTimeout(10000).setConnectTimeout(10000)
                .setCookieSpec(CookieSpecs.IGNORE_COOKIES).build();

        httpclient = HttpClients.custom().setConnectionManager(cm).disableAutomaticRetries()
                .setDefaultRequestConfig(requestConfig).build();

        apiKey = PropertyHolder.getString("$baidu.apiKey");
        secretKey = PropertyHolder.getString("$baidu.secretKey");

        if (StringUtils.isBlank(apiKey)) {
            LOG.error("'facepp.apiKey' is not configured");
        }
        if (StringUtils.isBlank(secretKey)) {
            LOG.error("'facepp.secretKey' is not configured");
        }
    }

    /**
     * 调用鉴权接口获取的token
     *
     * @return
     * @author WANGWEI
     */
    public String getAccessToken() {

        String url = "https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id="
                + apiKey + "&client_secret=" + secretKey;
        HttpPost httpPost = new HttpPost(url);
        httpPost.setConfig(BaiduClient.requestConfig);

        CloseableHttpResponse response = null;
        JsonHttpResponseHolder responseHolder = null;
        long s = System.currentTimeMillis();
        try {

            response = httpclient.execute(httpPost);
            int statusCode = response.getStatusLine().getStatusCode();
            String entityStr = EntityUtils.toString(response.getEntity(), "UTF-8");
            JSONObject obj = JSON.parseObject(entityStr);
            responseHolder = new JsonHttpResponseHolder(statusCode, obj);

            if (HttpStatus.SC_OK != responseHolder.getStatusCode()) {
                LOG.error("[Baidu AI Response]. statusCode=" + statusCode + "; responseEntity="
                        + entityStr);
            } else {
                if (LOG.isDebugEnabled()) {
                    LOG.debug("[Baidu AI Response]. statusCode=" + statusCode + "; responseEntity="
                            + entityStr);
                }
            }

        } catch (Exception e) {
            LOG.error("[Baidu AI FAIL]. fail to get access token.", e);
            throw new ExamCloudRuntimeException(e);
        } finally {
            IOUtils.closeQuietly(response);
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("[Baidu AI]. cost " + (System.currentTimeMillis() - s) + " ms.");
        }

        int statusCode = responseHolder.getStatusCode();
        if (HttpStatus.SC_OK != statusCode) {
            throw new StatusException("920", "[Baidu AI]. fail to get access token");
        }

        JSONObject respBody = responseHolder.getRespBody();
        String accessToken = respBody.getString("access_token");

        return accessToken;
    }

    /**
     * 活体检测
     *
     * @param imageUrl
     * @return
     * @author WANGWEI
     */
    public JsonHttpResponseHolder verifyFaceLiveness(String imageUrl) {

        if (LOG.isDebugEnabled()) {
            LOG.debug("[Baidu AI]. imageUrl=" + imageUrl);
        }

        String accessToken = getAccessToken();
        String url = "https://aip.baidubce.com/rest/2.0/face/v3/faceverify?access_token="
                + accessToken;

        HttpPost httpPost = new HttpPost(url);
        httpPost.setConfig(BaiduClient.requestConfig);
        httpPost.setHeader("Content-Type", "application/json");

        Map<String, String> params = Maps.newHashMap();

        params.put("image", imageUrl);
        params.put("image_type", "URL");

        List<Map<String, String>> list = Lists.newArrayList();
        list.add(params);

        httpPost.setEntity(new StringEntity(JsonUtil.toJson(list), "UTF-8"));

        CloseableHttpResponse response = null;
        JsonHttpResponseHolder responseHolder = null;
        long s = System.currentTimeMillis();
        try {

            response = httpclient.execute(httpPost);
            int statusCode = response.getStatusLine().getStatusCode();
            String entityStr = EntityUtils.toString(response.getEntity(), "UTF-8");
            JSONObject obj = JSON.parseObject(entityStr);
            responseHolder = new JsonHttpResponseHolder(statusCode, obj);

            if (HttpStatus.SC_OK != responseHolder.getStatusCode()) {
                LOG.error("[Baidu AI Response]. statusCode=" + statusCode + "; responseEntity="
                        + entityStr);
            } else {
                if (LOG.isDebugEnabled()) {
                    LOG.debug("[Baidu AI Response]. statusCode=" + statusCode + "; responseEntity="
                            + entityStr);
                }
            }

        } catch (Exception e) {
            LOG.error("[Baidu AI FAIL].", e);
            throw new ExamCloudRuntimeException(e);
        } finally {
            IOUtils.closeQuietly(response);
        }

        if (LOG.isDebugEnabled()) {
            LOG.debug("[Baidu AI]. imageUrl=" + imageUrl + "; cost "
                    + (System.currentTimeMillis() - s) + " ms.");
        }

        return responseHolder;
    }

    /**
     * 活体检测
     *
     * @param imageUrl
     * @return
     * @author WANGWEI
     */
    public JsonHttpResponseHolder verifyFaceLivenessUseBase64(String base64) {

        String accessToken = getAccessToken();
        String url = "https://aip.baidubce.com/rest/2.0/face/v3/faceverify?access_token="
                + accessToken;

        HttpPost httpPost = new HttpPost(url);
        httpPost.setConfig(BaiduClient.requestConfig);
        httpPost.setHeader("Content-Type", "application/json");

        Map<String, String> params = Maps.newHashMap();

        params.put("image", base64);
        params.put("image_type", "BASE64");

        List<Map<String, String>> list = Lists.newArrayList();
        list.add(params);

        httpPost.setEntity(new StringEntity(JsonUtil.toJson(list), "UTF-8"));

        CloseableHttpResponse response = null;
        JsonHttpResponseHolder responseHolder = null;
        try {

            response = httpclient.execute(httpPost);
            int statusCode = response.getStatusLine().getStatusCode();
            String entityStr = EntityUtils.toString(response.getEntity(), "UTF-8");
            JSONObject obj = JSON.parseObject(entityStr);
            responseHolder = new JsonHttpResponseHolder(statusCode, obj);

            if (HttpStatus.SC_OK != responseHolder.getStatusCode()) {
                LOG.error("[Baidu AI Response]. statusCode=" + statusCode + "; responseEntity="
                        + entityStr);
            } else {
                if (LOG.isDebugEnabled()) {
                    LOG.debug("[Baidu AI Response]. statusCode=" + statusCode + "; responseEntity="
                            + entityStr);
                }
            }

        } catch (Exception e) {
            LOG.error("[Baidu AI FAIL].", e);
            throw new ExamCloudRuntimeException(e);
        } finally {
            IOUtils.closeQuietly(response);
        }

        return responseHolder;
    }

    /**
     * 百度活体检测<br>
     * 优先使用主地址调用baidu活体检测接口<br>
     * 主地址无效时,使用备用地址调用baidu活体检测接口<br>
     * 备用地址也无效时,使用备用地址下载数据,使用下载数据的base64加密串调用baidu活体检测接口<br>
     *
     * @param imageUrl       主地址
     * @param backupImageUrl 备用地址
     * @return
     * @throws StatusException code为901,902,903表示图片地址无效
     * @author WANGWEI
     */
    public JsonHttpResponseHolder verifyFaceLiveness(String imageUrl, String backupImageUrl)
            throws StatusException {

        if (LOG.isDebugEnabled()) {
            LOG.debug("[Face++]. imageUrl=" + imageUrl + "; backupImageUrl=" + backupImageUrl);
        }

        JsonHttpResponseHolder responseHolder = null;

        boolean exceptionWhenUsingImageUrl = false;
        boolean exceptionWhenUsingBackupImageUrl = false;

        try {
            responseHolder = verifyFaceLiveness(imageUrl);
        } catch (ExamCloudRuntimeException e) {
            exceptionWhenUsingImageUrl = true;
        }

        if (exceptionWhenUsingImageUrl) {
            try {
                responseHolder = verifyFaceLiveness(backupImageUrl);
            } catch (ExamCloudRuntimeException e) {
                exceptionWhenUsingBackupImageUrl = true;
            }
        } else {
            long errCode = responseHolder.getRespBody().getLong("error_code");

            if (0 == errCode) {
                return responseHolder;
            }

            if (retry(errCode)) {
                try {
                    responseHolder = verifyFaceLiveness(backupImageUrl);
                } catch (ExamCloudRuntimeException e) {
                    exceptionWhenUsingBackupImageUrl = true;
                }
            }

        }

        long errCode = responseHolder.getRespBody().getLong("error_code");

        if (0 == errCode) {
            return responseHolder;
        }

        if (exceptionWhenUsingBackupImageUrl || retry(errCode)) {
            HttpGet get = new HttpGet(backupImageUrl);
            get.setConfig(BaiduClient.requestConfig);
            CloseableHttpResponse response = null;
            String imageBase64 = null;
            long s = System.currentTimeMillis();
            try {
                response = httpclient.execute(get);

                if (HttpStatus.SC_OK != response.getStatusLine().getStatusCode()) {
                    throw new StatusException("901",
                            "fail to download image file. url=" + backupImageUrl);
                }

                byte[] byteArray = EntityUtils.toByteArray(response.getEntity());
                if (100 > byteArray.length) {
                    throw new StatusException("902", "invalid image size. url=" + backupImageUrl);
                }

                imageBase64 = Base64.encodeBase64String(byteArray);

            } catch (StatusException e) {
                LOG.error("fail to download image file. url=" + backupImageUrl, e);
                throw e;
            } catch (Exception e) {
                LOG.error("fail to download image file. url=" + backupImageUrl, e);
                throw new StatusException("903", "fail to download file. url=" + backupImageUrl, e);
            } finally {
                IOUtils.closeQuietly(response);
            }

            if (LOG.isDebugEnabled()) {
                LOG.debug("download image file successfully; url=" + backupImageUrl + "; cost "
                        + (System.currentTimeMillis() - s) + " ms.");
            }

            responseHolder = verifyFaceLivenessUseBase64(imageBase64);
        }

        return responseHolder;
    }

    /**
     * 是否重试
     *
     * @param errCode
     * @return
     * @author WANGWEI
     */
    private boolean retry(long errCode) {
        return 222204 == errCode || 222013 == errCode;
    }

}
