package cn.com.qmth.examcloud.web.redis;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.concurrent.TimeUnit;

/**
 * redis client
 *
 * @author WANGWEI
 * @date 2019年2月22日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public final class SimpleRedisClient implements RedisClient {

    private static final Logger LOG = LoggerFactory.getLogger(SimpleRedisClient.class);

    private RedisTemplate<String, Object> redisTemplate;

    private boolean enable = true;

    public SimpleRedisClient(RedisTemplate<String, Object> redisTemplate) {
        super();
        this.redisTemplate = redisTemplate;
    }

    private void beforeMethod() {
        if (!enable) {
            throw new ExamCloudRuntimeException("RedisClient is not enabled");
        }
    }

    private void afterMethod(String content, long startTimeMillis) {
        long cost = System.currentTimeMillis() - startTimeMillis;
        if (cost > 1000) {
            LOG.warn("{} cost {} ms", content, cost);
        } else {
            LOG.debug("{} cost {} ms", content, cost);
        }
    }

    @Override
    public void set(String key, Object value) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.opsForValue().set(key, value);

        afterMethod("set " + key, s);
    }

    @Override
    public void set(String key, Object value, int timeout) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.SECONDS);

        afterMethod("set " + key, s);
    }

    @Override
    public void expire(String key, int timeout) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.expire(key, timeout, TimeUnit.SECONDS);

        afterMethod("expire " + key, s);
    }

    /**
     * 方法注释
     *
     * @param key
     * @param timeout
     * @param unit
     * @author WANGWEI
     */
    @Override
    public void expire(String key, final long timeout, final TimeUnit unit) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.expire(key, timeout, unit);

        afterMethod("expire " + key, s);
    }

    @Override
    public <T> T get(String key, Class<T> c, int timeout) {
        long s = System.currentTimeMillis();
        beforeMethod();
        Object object = redisTemplate.opsForValue().get(key);
        @SuppressWarnings("unchecked")
        T t = (T) object;
        expire(key, timeout);

        afterMethod("get " + key, s);
        return t;
    }

    @Override
    public <T> T get(String key, Class<T> c) {
        long s = System.currentTimeMillis();
        beforeMethod();
        Object object = redisTemplate.opsForValue().get(key);
        @SuppressWarnings("unchecked")
        T t = (T) object;

        afterMethod("get " + key, s);
        return t;
    }


    @Override
    public <T> T get(String key, String subKey, Class<T> c) {
        long s = System.currentTimeMillis();
        beforeMethod();
        Object object = redisTemplate.opsForHash().get(key, subKey);
        @SuppressWarnings("unchecked")
        T t = (T) object;

        afterMethod("hash get " + key + " " + subKey, s);
        return t;
    }

    @Override
    public void delete(String key) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.expire(key, 0, TimeUnit.SECONDS);

        afterMethod("delete " + key, s);
    }

    @Override
    public void convertAndSend(String channel, Object message) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.convertAndSend(channel, message);

        afterMethod("convertAndSend " + channel, s);
    }

    @Override
    public Boolean setIfAbsent(String key, String value, int timeout) {
        long s = System.currentTimeMillis();
        beforeMethod();
        Boolean b = redisTemplate.opsForValue().setIfAbsent(key, value, timeout, TimeUnit.SECONDS);

        afterMethod("setIfAbsent " + key, s);
        return b;
    }

    @Override
    public boolean isEnable() {
        return enable;
    }

    @Override
    public void setEnable(boolean enable) {
        this.enable = enable;
    }

    @Override
    public Long getExpire(String key, TimeUnit timeUnit) {
        return redisTemplate.getExpire(key, timeUnit);
    }

    @Override
    public void set(String key, String subKey, Object value, int timeout) {
        long s = System.currentTimeMillis();
        beforeMethod();
        set(key, subKey, value);
        expire(key, timeout);
    }

    @Override
    public void delete(String key, String subKey) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.opsForHash().delete(key, subKey);

        afterMethod("hash delete " + key + " " + subKey, s);
    }

    @Override
    public void set(String key, String subKey, Object value) {
        long s = System.currentTimeMillis();
        beforeMethod();
        redisTemplate.opsForHash().put(key, subKey, value);

        afterMethod("hash put " + key + " " + subKey, s);
    }

}
