package cn.com.qmth.examcloud.web.security;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.bean.Role;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.api.commons.security.enums.RoleMeta;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

public class DataRuleInterceptor extends HandlerInterceptorAdapter {

    private static final Logger LOG = LoggerFactory.getLogger(DataRuleInterceptor.class);

    private ResourceManager resourceManager;

    public DataRuleInterceptor(ResourceManager resourceManager) {
        this.resourceManager = resourceManager;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        try {
            DataRule dataRule = null;
            if (handler instanceof HandlerMethod) {
                HandlerMethod handlerMethod = (HandlerMethod) handler;
                dataRule = handlerMethod.getMethodAnnotation(DataRule.class);
            }

            if (dataRule == null) {
                return true;
            }

            boolean isEnable = PropertyHolder.getBoolean("examcloud.data.rule.enable", false);
            if (!isEnable) {
                // 未启用数据权限
                LOG.info("examcloud.data.rule.enable = false");
                return true;
            }

            User accessUser = (User) request.getAttribute(HttpServletRequestAttribute.$_ACCESS_USER.name());
            if (accessUser == null) {
                return true;
            }

            boolean isAdmimUser = this.hasAnyRoles(accessUser, RoleMeta.SUPER_ADMIN, RoleMeta.ORG_ADMIN);
            boolean isStudent = UserType.STUDENT == accessUser.getUserType();

            for (DataRuleType dataRuleType : dataRule.type()) {
                UserDataRule userDataRule;

                if (isAdmimUser || isStudent) {
                    // “超管、机构管理员、学生” 暂例外
                    userDataRule = new UserDataRule();
                    userDataRule.setGlobalStatus(true);
                    userDataRule.setRefIds(new HashSet<>());
                } else {
                    userDataRule = resourceManager.loadUserDataRule(accessUser.getUserId(), dataRuleType);
                    LOG.info(String.format("userId = %s, type = %s, %s", accessUser.getUserId(), dataRuleType, userDataRule));
                }

                String attributeName = HttpServletRequestAttribute.$_USER_DATA_RULE.name() + "@" + dataRuleType.name();
                request.setAttribute(attributeName, userDataRule);
            }
        } catch (Exception e) {
            LOG.error(e.getMessage(), e);
        }

        return true;
    }

    /**
     * 判断是否包含任一角色
     */
    private boolean hasAnyRoles(User accessUser, RoleMeta... roles) {
        List<Role> roleList = accessUser.getRoleList();
        if (CollectionUtils.isEmpty(roleList)) {
            return false;
        }

        Set<String> roleCodes = roleList.stream().map(e -> e.getRoleCode()).collect(Collectors.toSet());
        for (RoleMeta role : roles) {
            if (roleCodes.contains(role.name())) {
                LOG.info(String.format("userId = %s, role = %s", accessUser.getUserId(), roleCodes));
                return true;
            }
        }

        return false;
    }

}
