package cn.com.qmth.examcloud.web.security;

import cn.com.qmth.examcloud.api.commons.CloudService;
import cn.com.qmth.examcloud.api.commons.EnterpriseService;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ApiInfo;
import cn.com.qmth.examcloud.web.support.ServletUtil;
import cn.com.qmth.examcloud.web.support.StatusResponse;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 请求鉴权
 *
 * @author WANGWEI
 * @date 2018年5月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class RequestPermissionInterceptor implements HandlerInterceptor {

    private static final Logger LOG = LoggerFactory.getLogger(RequestPermissionInterceptor.class);

    private ResourceManager resourceManager;

    private RedisClient redisClient;

    /**
     * 构造函数
     *
     * @param resourceManager
     * @param redisClient
     */
    public RequestPermissionInterceptor(ResourceManager resourceManager, RedisClient redisClient) {
        super();
        this.resourceManager = resourceManager;
        this.redisClient = redisClient;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
                             Object handler) throws Exception {
        // if (LOG.isDebugEnabled()) {
        //     LOG.debug(String.format("[%s] - %s", request.getMethod(), request.getServletPath()));
        // }

        ApiInfo apiInfo = (ApiInfo) request
                .getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

        if (null != apiInfo) {
            Class<?> ctrClass = apiInfo.getBeanType();
            if (CloudService.class.isAssignableFrom(ctrClass)
                    || EnterpriseService.class.isAssignableFrom(ctrClass)) {
                return true;
            }
        }

        String mapping = (String) request
                .getAttribute(HttpServletRequestAttribute.$_MAPPING.name());

        boolean naked = resourceManager.isNaked(apiInfo, mapping);
        if (naked) {
            return true;
        }

        String key = null;
        String token = null;
        String kt = request.getHeader("user_token");
        if (StringUtils.isNotBlank(kt)) {
            String[] arr = kt.split(":");
            if (null != arr && 2 == arr.length) {
                key = arr[0];
                token = arr[1];
            }
        } else {
            key = request.getHeader("key");
            token = request.getHeader("token");
        }

        if (StringUtils.isBlank(key) && StringUtils.isBlank(token)) {
            key = request.getParameter("$key");
            token = request.getParameter("$token");
        }

        if (StringUtils.isBlank(key)) {
            response.setStatus(HttpStatus.FORBIDDEN.value());
            ServletUtil.returnJson(new StatusResponse("403", "key is blank."), response);
            return false;
        }
        if (StringUtils.isBlank(token)) {
            response.setStatus(HttpStatus.FORBIDDEN.value());
            ServletUtil.returnJson(new StatusResponse("403", "token is blank."), response);
            return false;
        }

        User user = redisClient.get(key, User.class);

        if (null == user) {
            response.setStatus(HttpStatus.FORBIDDEN.value());
            ServletUtil.returnJson(new StatusResponse("403", "no login."), response);
            return false;
        } else if (!token.equals(user.getToken())) {
            response.setStatus(HttpStatus.FORBIDDEN.value());
            ServletUtil.returnJson(new StatusResponse("403", "token is wrong."), response);
            return false;
        }

        redisClient.expire(key, user.getSessionTimeout());

        ThreadContext.put("CALLER", key);

        if (!resourceManager.hasPermission(user, apiInfo, mapping)) {
            response.setStatus(HttpStatus.METHOD_NOT_ALLOWED.value());
            ServletUtil.returnJson(new StatusResponse("405", "no permission."), response);
            return false;
        }

        request.setAttribute(HttpServletRequestAttribute.$_ACCESS_USER.name(), user);
        request.setAttribute("$kt", key + ":" + token);

        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
                           ModelAndView modelAndView) throws Exception {
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
                                Object handler, Exception ex) throws Exception {
    }

}
